#include "amarok14datagetter.h"

Amarok14DataGetter::Amarok14DataGetter(){
    m_timer = new QTimer();
    m_process_path = new QProcess();
    m_process_artist = new QProcess();
    m_process_title = new QProcess();
    m_process_track = new QProcess();
    m_process_album = new QProcess();
    m_process_length = new QProcess();
    m_data = new QSettings(QSettings::defaultFormat(), QSettings::UserScope, "qutim", ".nowlisteningamarokdata");
    m_meta_info = new TrackMetaInfo();
    connect (m_timer, SIGNAL(timeout()), this, SLOT(timeOut()));
    connect (m_process_path, SIGNAL(readyReadStandardOutput()), this, SLOT(readPath()));
    connect (m_process_artist, SIGNAL(readyReadStandardOutput()), this, SLOT(readArtist()));
    connect (m_process_title, SIGNAL(readyReadStandardOutput()), this, SLOT(readTitle()));
    connect (m_process_album, SIGNAL(readyReadStandardOutput()), this, SLOT(readAlbum()));
    connect (m_process_length, SIGNAL(readyReadStandardOutput()), this, SLOT(readLength()));
    connect (m_process_track, SIGNAL(readyReadStandardOutput()), this, SLOT(readTrack()));
    m_timer->start(5000);
}

Amarok14DataGetter::~Amarok14DataGetter(){
    m_timer->stop();
    m_process_path->kill();
    m_data->setValue("URI", "");
    m_data->setValue("Title","");
    m_data->setValue("Artist","");
    m_data->setValue("Album","");
    m_data->setValue("Length","");
    m_data->setValue("Track","");
    delete m_timer;
    delete m_process_path;
    delete m_data;
    delete m_meta_info;
    delete m_process_artist;
    delete m_process_title;
    delete m_process_track;
    delete m_process_album;
    delete m_process_length;
}

void Amarok14DataGetter::timeOut(){
    m_process_path->start("dcop amarok player path");
}

void Amarok14DataGetter::readPath(){
    m_meta_info->uri = m_process_path->readAllStandardOutput().replace("\n","");
    if ( m_data->value("URI","").toString() != m_meta_info->uri){
        m_data->setValue("URI", m_meta_info->uri);
        m_timer->stop();
        m_process_artist->start("dcop amarok player artist");
    }
}

void Amarok14DataGetter::readArtist(){
    m_meta_info->artist = m_process_artist->readAllStandardOutput().replace("\n","");
    m_data->setValue("Artist", m_meta_info->artist);
    m_process_title->start("dcop amarok player title");
}


void Amarok14DataGetter::readTitle(){
    m_meta_info->title = m_process_title->readAllStandardOutput().replace("\n","");
    m_data->setValue("Title", m_meta_info->title);
    m_process_album->start("dcop amarok player album");
}

void Amarok14DataGetter::readAlbum(){
    m_meta_info->album = m_process_album->readAllStandardOutput().replace("\n","");
    m_data->setValue("Album", m_meta_info->album);
    m_process_length->start("dcop amarok player totalTime");
}

void Amarok14DataGetter::readLength(){
    m_meta_info->time_min = m_process_length->readAllStandardOutput().replace("\n","");
    m_data->setValue("Length", m_meta_info->time_min);
    m_process_track->start("dcop amarok player track");
}

void Amarok14DataGetter::readTrack(){
    m_meta_info->number = m_process_track->readAllStandardOutput().replace("\n","");
    m_data->setValue("Track", m_meta_info->number);
    m_timer->start(5000);
}

QString Amarok14DataGetter::getURI(){
    return (m_meta_info->uri);
}

QString Amarok14DataGetter::getArtist(){
    return (m_meta_info->artist);
}

QString Amarok14DataGetter::getTitle(){
    return (m_meta_info->title);
}

TrackMetaInfo Amarok14DataGetter::getTrackMetaInfo(){
    TrackMetaInfo meta_info;
    meta_info.uri = m_data->value("URI","").toString();
    meta_info.artist = m_data->value("Artist","").toString();
    meta_info.title = m_data->value("Title","").toString();
    meta_info.album = m_data->value("Album","").toString();
    meta_info.time_min = m_data->value("Length","").toString();
    meta_info.time_msec = 0;
    meta_info.number = m_data->value("Track","").toString();
    meta_info.file_name = meta_info.uri.section(
                            "", meta_info.uri.lastIndexOf( "/", -1 ) + 2 ).
                            replace( "%20", " " );
    return (meta_info);
}
