#include "mprisdatagetter.h"

MPRISDataGetter::MPRISDataGetter(QString player_name, int timer_period){
    m_player_name = player_name;
    if (m_player_name == "Amarok 2"){
        m_dbus_interface = new QDBusInterface("org.kde.amarok", "/Player");
        m_uri_call_method = "location";
        m_length_call_method = "mtime";
        m_artist_call_method = "artist";
        m_title_call_method = "title";
        m_album_call_method = "album";
        m_track_call_method = "track";
        m_timer = new QTimer();
        connect (m_timer, SIGNAL(timeout()), this, SLOT(isTrackChanged()));
        connect (m_timer, SIGNAL(timeout()), this, SLOT(checkPlayerStatus()));
        m_timer->start(timer_period * 1000);
    }
    else if (m_player_name == "QMMP"){
        m_dbus_interface = new QDBusInterface("org.mpris.qmmp", "/Player", "org.freedesktop.MediaPlayer",
                                                QDBusConnection::sessionBus(), this);
        m_uri_call_method = "location";
        m_length_call_method = "mtime";
        m_artist_call_method = "artist";
        m_title_call_method = "title";
        m_album_call_method = "album";
        m_track_call_method = "track";
        connect(m_dbus_interface, SIGNAL(TrackChange(QVariantMap)), this, SLOT(trackChanged(QVariantMap)));
        m_timer = new QTimer();
        connect (m_timer, SIGNAL(timeout()), this, SLOT(checkPlayerStatus()));
        m_timer->start(timer_period * 1000);

    }
    else if (m_player_name == "Audacious"){
        m_dbus_interface = new QDBusInterface("org.mpris.audacious", "/Player");
        m_uri_call_method = "URI";
        m_length_call_method = "length";
        m_artist_call_method = "artist";
        m_title_call_method = "title";
        m_album_call_method = "album";
        m_track_call_method = "track";
    }
    else if ( m_player_name == "Rhythmbox" ){
        m_dbus_interface = new QDBusInterface("org.gnome.Rhythmbox", "/org/gnome/Rhythmbox/Player",
                                              "org.gnome.Rhythmbox.Player", QDBusConnection::sessionBus(), this);
        m_uri_call_method = "location";
        m_length_call_method = "duration";
        m_artist_call_method = "artist";
        m_title_call_method = "title";
        m_album_call_method = "album";
        m_track_call_method = "track-number";

        connect (m_dbus_interface, SIGNAL(playingUriChanged(QString)), this, SLOT(trackChanged(QString)));
        connect (m_dbus_interface, SIGNAL(playingChanged(bool)), this, SLOT(playerStatusChanged(bool)));
    }
    else if (m_player_name == "VLC"){
        m_dbus_interface = new QDBusInterface("org.mpris.vlc", "/Player");
        m_uri_call_method = "location";
        m_length_call_method = "length";
        m_artist_call_method = "artist";
        m_title_call_method = "title";
        m_album_call_method = "album";
        m_track_call_method = "track";

        m_timer = new QTimer();
        connect (m_timer, SIGNAL(timeout()), this, SLOT(isTrackChanged()));
        connect (m_timer, SIGNAL(timeout()), this, SLOT(checkPlayerStatus()));
        m_timer->start(timer_period * 1000);
    }
    else if (m_player_name == "Songbird (Linux)"){
        m_dbus_interface = new QDBusInterface("org.mpris.songbird", "/Player");
        m_uri_call_method = "location";
        m_length_call_method = "mtime";
        m_artist_call_method = "artist";
        m_title_call_method = "title";
        m_album_call_method = "album";
        m_track_call_method = "tracknumber";
    }
    m_was_stopped = isPlayingStopped();
}

MPRISDataGetter::~MPRISDataGetter(){
    m_timer->stop();
    delete m_dbus_interface;
    m_dbus_interface = NULL;
    delete m_timer;
}


QString MPRISDataGetter::getURI(){
    QString uri = "";
    if (m_player_name != "Rhythmbox"){
        QDBusReply<QVariantMap> reply = m_dbus_interface->call("GetMetadata");
        uri = reply.value().value(m_uri_call_method).toString();
    }
    else if ( m_player_name == "Rhythmbox" ){
        QDBusReply<QString> value = m_dbus_interface->call("getPlayingUri");
        uri = value;
    }
    return uri;
}

QString MPRISDataGetter::getTitle(){
    QString title = "";
    if (m_player_name != "Rhythmbox"){
        QDBusReply<QVariantMap> reply = m_dbus_interface->call("GetMetadata");
        title = reply.value().value(m_title_call_method).toString();
    }
    else if (m_player_name == "Rhythmbox"){
        QDBusInterface dbus_interface("org.gnome.Rhythmbox", "/org/gnome/Rhythmbox/Shell");
        QDBusReply<QVariantMap> reply = dbus_interface.call("getSongProperties", getURI());
        title = reply.value().value(m_title_call_method).toString();
    }
    return title;
}

QString MPRISDataGetter::getArtist(){
    QString artist = "";
    if (m_player_name != "Rhythmbox"){
        QDBusReply<QVariantMap> reply = m_dbus_interface->call("GetMetadata");
        artist = reply.value().value(m_artist_call_method).toString();
    }
    else if (m_player_name == "Rhythmbox"){
        QDBusInterface dbus_interface("org.gnome.Rhythmbox", "/org/gnome/Rhythmbox/Shell");
        QDBusReply<QVariantMap> reply = dbus_interface.call("getSongProperties", getURI());
        artist = reply.value().value(m_artist_call_method).toString();
    }
    return artist;
}


TrackMetaInfo MPRISDataGetter::getTrackMetaInfo(){
    TrackMetaInfo meta_info;
    if (m_player_name != "Rhythmbox"){
        QDBusReply<QVariantMap> reply = m_dbus_interface->call("GetMetadata");
        meta_info.artist = reply.value().value(m_artist_call_method).toString();
        meta_info.title = reply.value().value(m_title_call_method).toString();
        meta_info.album = reply.value().value(m_album_call_method).toString();
        meta_info.time_msec = reply.value().value(m_length_call_method).toInt();
        int leng_sec = meta_info.time_msec / 1000;
        QVariant min (leng_sec/60);
        QVariant sec (leng_sec - (leng_sec/60)*60);
        meta_info.time_min = "";
        meta_info.time_min += ( sec.toInt() < 10) ?  min.toString() + ":0" + sec.toString()
                                               : min.toString() + ":" + sec.toString();
        meta_info.number = reply.value().value(m_track_call_method).toString();
        meta_info.uri = reply.value().value(m_uri_call_method).toString();
        meta_info.file_name = meta_info.uri.section(
                            "", meta_info.uri.lastIndexOf( "/", -1 ) + 2 ).
                            replace( "%20", " " );
    }
    else if (m_player_name == "Rhythmbox"){
        QDBusInterface dbus_interface("org.gnome.Rhythmbox", "/org/gnome/Rhythmbox/Shell");
        QDBusReply<QVariantMap> reply = dbus_interface.call("getSongProperties", getURI());
        meta_info.artist = reply.value().value(m_artist_call_method).toString();
        meta_info.title = reply.value().value(m_title_call_method).toString();
        meta_info.album = reply.value().value(m_album_call_method).toString();
        meta_info.time_msec = reply.value().value(m_length_call_method).toInt()*1000;
        int leng_sec = meta_info.time_msec / 1000;
        QVariant min (leng_sec/60);
        QVariant sec (leng_sec - (leng_sec/60)*60);
        meta_info.time_min = "";
        meta_info.time_min += ( sec.toInt() < 10) ?  min.toString() + ":0" + sec.toString()
                                               : min.toString() + ":" + sec.toString();
        meta_info.number = reply.value().value(m_track_call_method).toString();
        meta_info.uri = reply.value().value(m_uri_call_method).toString();
        meta_info.file_name = meta_info.uri.section(
                            "", meta_info.uri.lastIndexOf( "/", -1 ) + 2 ).
                            replace( "%20", " " );
    }
    return meta_info;
}

TrackMetaInfo MPRISDataGetter::getTrackMetaInfo(QVariantMap reply){
    TrackMetaInfo meta_info;
    meta_info.artist = reply.value(m_artist_call_method).toString();
    meta_info.title = reply.value(m_title_call_method).toString();
    meta_info.album = reply.value(m_album_call_method).toString();
    meta_info.time_msec = reply.value(m_length_call_method).toInt()*1000;
    int leng_sec = meta_info.time_msec / 1000;
    QVariant min (leng_sec/60);
    QVariant sec (leng_sec - (leng_sec/60)*60);
    meta_info.time_min = "";
    meta_info.time_min += ( sec.toInt() < 10) ?  min.toString() + ":0" + sec.toString()
                                               : min.toString() + ":" + sec.toString();
    meta_info.number = reply.value(m_track_call_method).toString();
    meta_info.uri = reply.value(m_uri_call_method).toString();
    meta_info.file_name = meta_info.uri.section(
                            "", meta_info.uri.lastIndexOf( "/", -1 ) + 2 ).
                            replace( "%20", " " );
    return meta_info;
}

TrackMetaInfo MPRISDataGetter::getTrackMetaInfo(QDBusReply<QVariantMap> reply){
    TrackMetaInfo meta_info;
    meta_info.artist = reply.value().value(m_artist_call_method).toString();
    meta_info.title = reply.value().value(m_title_call_method).toString();
    meta_info.album = reply.value().value(m_album_call_method).toString();
    meta_info.time_msec = reply.value().value(m_length_call_method).toInt()*1000;
    int leng_sec = meta_info.time_msec / 1000;
    QVariant min (leng_sec/60);
    QVariant sec (leng_sec - (leng_sec/60)*60);
    meta_info.time_min = "";
    meta_info.time_min += ( sec.toInt() < 10) ?  min.toString() + ":0" + sec.toString()
                                               : min.toString() + ":" + sec.toString();
    meta_info.number = reply.value().value(m_track_call_method).toString();
    meta_info.uri = reply.value().value(m_uri_call_method).toString();
    meta_info.file_name = meta_info.uri.section(
                            "", meta_info.uri.lastIndexOf( "/", -1 ) + 2 ).
                            replace( "%20", " " );
    return meta_info;
}

bool MPRISDataGetter::isPlayingStopped(){

    if (m_player_name == "QMMP" || m_player_name == "Amarok 2" || m_player_name == "VLC") {
        QDBusReply<int> position_1;
        if (m_dbus_interface != NULL) position_1 = m_dbus_interface->call("PositionGet");
        else return true;
        QTest::qWait(1000);
        QDBusReply<int> position_2;
        if (m_dbus_interface != NULL) position_2 = m_dbus_interface->call("PositionGet");
        else return true;
        if (position_2 - position_1) return false;
        else return true;
    }
    else if (m_player_name == "Rhythmbox"){
        QDBusReply<bool> is_playing;
        if (m_dbus_interface != NULL) is_playing = m_dbus_interface->call("getPlaying");
        else return true;
        return !is_playing;
    }

    return true;
}

void MPRISDataGetter::trackChanged(QVariantMap track_info){
    emit trackChanged(getTrackMetaInfo(track_info));
}

void MPRISDataGetter::checkPlayerStatus(){
    if (isPlayingStausChanged()){
        qDebug()<<"!!!!! status changed";
        emit playingStoppedStatusChanged(m_was_stopped);
    }
}

bool MPRISDataGetter::isPlayingStausChanged(){
    if (isPlayingStopped() == m_was_stopped) return false;
    else {
        m_was_stopped = !m_was_stopped;
        return true;
    }
}

void MPRISDataGetter::isTrackChanged(){
    QString temp;
    temp = getURI();
    if ( temp != m_current_uri ){
        m_current_uri = temp;
        m_current_artist = getArtist();
        m_current_title = getTitle();
        emit trackChanged(getTrackMetaInfo());
    }
    else {
        temp = getTitle();
        if ( temp != m_current_title ){
            m_current_title = temp;
            m_current_uri = getURI();
            m_current_artist = getArtist();
            emit trackChanged(getTrackMetaInfo());
        }
        else {
            temp = getArtist();
            if ( temp != m_current_artist ){
                m_current_artist = temp;
                m_current_uri = getURI();
                m_current_title = getTitle();
                emit trackChanged(getTrackMetaInfo());
            }
        }
    }
}

void MPRISDataGetter::trackChanged(int num){
    emit trackChanged(getTrackMetaInfo());
}

void MPRISDataGetter::trackChanged(QString temp){
    emit trackChanged(getTrackMetaInfo());
}

void MPRISDataGetter::playerStatusChanged(bool is_playing){
    emit playingStoppedStatusChanged(!is_playing);
}
