#include "nowlisteningpluginsystem.h"

bool NowListeningPluginSystem :: init ( PluginSystemInterface *plugin_system ){

    PluginInterface::init ( plugin_system );
    m_plugin_system = plugin_system;
    m_plugin_icon = new QIcon ( ":/images/plugin_logo.png" );

    QTextCodec::setCodecForCStrings( QTextCodec::codecForName("UTF-8") );

    winampDataGetter = 0;
    m_settings_loaded = false;
    m_plugin_active = false;
    m_current_icq_status = new QHash<QString,int>();
    m_is_icq_account_active = false;

    m_event_online = m_plugin_system->registerEventHandler("Core/Protocol/AccountIsOnline", this);
    m_event_init = m_plugin_system->registerEventHandler("Core/Layers/Initialized", this);
    m_event_jabber = m_plugin_system->registerEventHandler("Jabber/Account/Tune/Change", this);
    m_event_icq = m_plugin_system->registerEventHandler("ICQ/Account/XStatus/Set", this);
    m_event_mrim_status_changed = m_plugin_system->registerEventHandler("MRIM/Account/Status/Changed", this);
    m_event_mrim_status_set = m_plugin_system->registerEventHandler("MRIM/Account/Status/Change", this);
    m_event_mrim_status_set_text = m_plugin_system->registerEventHandler("MRIM/Account/Status/ChangeText", this);

    return true;

}

void NowListeningPluginSystem :: processEvent ( Event &event ){

    if ( event.id == m_event_init ){
        loadSettings();
        m_settings_loaded = true;
        activatePlugin();
    }

    if (event.id == m_event_mrim_status_changed){
        writeCurrentMrimStatus(*(QString*)(event.args.at(0)), *(qint32*)(event.args.at(1)), *(QString*)(event.args.at(2)), *(QString*)(event.args.at(3)), *(QString*)(event.args.at(4)));
        setMrimXStatus(getMrimAccountIndexInList(*(QString*)(event.args.at(0))));
    }
}

QWidget* NowListeningPluginSystem :: settingsWidget(){

    m_settings_form = new SettingsUI(0, 0, m_profile_name);
    return m_settings_form;

}

void NowListeningPluginSystem :: removeSettingsWidget(){

    delete m_settings_form;

}

void NowListeningPluginSystem :: setProfileName ( const QString &profile_name ){

    m_profile_name = profile_name;

    if (m_plugin_active) m_manage_plugin = new QAction(QIcon(":/images/plugin_logo.png"), tr("Activate Now Listening"), this);
    else m_manage_plugin = new QAction(QIcon(":/images/plugin_logo.png"), tr("Deactivate Now Listening"), this);
    m_plugin_system->registerMainMenuAction(m_manage_plugin);
    connect(m_manage_plugin, SIGNAL(triggered()), this, SLOT(managePluginClicked()));

}

QString NowListeningPluginSystem :: name(){

    return "Now Listening";

}

QString NowListeningPluginSystem :: description(){

    return "Plugin sets your status message according to artist and title of the track, you are currently listening";
}

QString NowListeningPluginSystem :: type(){

    return "other";

}

void NowListeningPluginSystem :: release(){

}

QIcon* NowListeningPluginSystem :: icon(){
    return m_plugin_icon;
}

void NowListeningPluginSystem :: saveSettings(){

    m_settings_form->saveSettings();
    deactivatePlugin();
    m_icq_account_settings.clear();
    m_jabber_account_settings.clear();
    m_mrim_account_settings.clear();
    loadSettings();
    activatePlugin();

}

void NowListeningPluginSystem :: loadSettings(){

    m_current_icq_status->clear();
    m_icq_account_settings.clear();
    m_jabber_account_settings.clear();
    m_mrim_account_settings.clear();
    QSettings* protocol_settings = new QSettings(QSettings::defaultFormat(), QSettings::UserScope, "qutim/qutim."+m_profile_name, "icqsettings");
    QSettings* settings = new QSettings(QSettings::defaultFormat(), QSettings::UserScope, "qutim/qutim."+m_profile_name, "NowListening");
    m_accounts = protocol_settings->value("accounts/list").toStringList();
    for (int i = 0; i < m_accounts.size(); i++){
        IcqAccountSettings* temp = new IcqAccountSettings;
        temp->name = m_accounts[i];
        QSettings* accountsettings = new QSettings(QSettings::defaultFormat(), QSettings::UserScope,
                                                   "qutim/qutim."+m_profile_name+"/ICQ."+temp->name, "accountsettings");
        m_current_icq_status->insert(temp->name, accountsettings->value("xstatus/index",11).toInt());
        delete accountsettings;
        temp->status = settings->value(temp->name+"/status", 0).toInt();
        temp->message_mask_1 = settings->value(temp->name+"/message_mask_1", "Now playing: %artist - %title").toString();
        temp->message_mask_2 = settings->value(temp->name+"/message_mask_2", "%artist - %title").toString();
        m_icq_account_settings.append(*temp);
        delete temp;
    }
    delete protocol_settings;
    protocol_settings = new QSettings(QSettings::defaultFormat(), QSettings::UserScope, "qutim/qutim."+m_profile_name, "jabbersettings");
    QStringList acc_temp = protocol_settings->value("accounts/list").toStringList();
    m_accounts += acc_temp;
    for (int i = 0; i < acc_temp.size(); i++){
        JabberAccountSettings* temp = new JabberAccountSettings;
        temp->name = acc_temp[i];
        temp->status = settings->value(temp->name+"/status", 0).toInt();
        temp->message_mask = settings->value(temp->name+"/message", "artist title").toString();
        m_jabber_account_settings.append(*temp);
        delete temp;
    }
    delete protocol_settings;
    protocol_settings = new QSettings(QSettings::defaultFormat(), QSettings::UserScope, "qutim/qutim."+m_profile_name, "mrimsettings");
    acc_temp.clear();
    acc_temp = protocol_settings->value("accounts/list").toStringList();
    m_accounts += acc_temp;
    for (int i = 0; i < acc_temp.size(); i++){
        MrimAccountSettings* temp = new MrimAccountSettings;
        temp->name = acc_temp[i];
        temp->status = settings->value(temp->name+"/status", 0).toInt();
        temp->mode = settings->value(temp->name+"/mode", 2).toInt();
        temp->message_mask_1 = settings->value(temp->name+"/message_mask_1", "Now playing: %artist - %title").toString();
        temp->message_mask_2 = settings->value(temp->name+"/message_mask_2", "%artist - %title").toString();
        m_mrim_account_settings.append(*temp);
        delete temp;
    }
    delete protocol_settings;

    m_global_settings.global = settings->value("Global/global", 0).toInt();
    m_global_settings.icq_status = settings->value("Global/icq_status", 0).toInt();
    m_global_settings.mrim_status = settings->value("Global/mrim_status", 0).toInt();
    m_global_settings.jabber_status = settings->value("Global/jabber_status", 0).toInt();
    m_global_settings.mrim_mode = settings->value("Global/mrim_mode", 0).toInt();
    m_global_settings.icq_message_mask_1 = settings->value("Global/icq_message_mask_1", "Now playing: %artist - %title").toString();
    m_global_settings.icq_message_mask_2 = settings->value("Global/icq_message_mask_2", "%artist - %title").toString();
    m_global_settings.mrim_message_mask_1 = settings->value("Global/mrim_message_mask_1", "Now playing: %artist - %title").toString();
    m_global_settings.mrim_message_mask_2 = settings->value("Global/mrim_message_mask_2", "%artist - %title").toString();
    m_global_settings.jabber_message = settings->value("Global/jabber_message", "artist title").toString();

    m_global_mode = settings->value("Main/global_mode", true).toBool();
    m_player_name = settings->value("Main/player", "Amarok 2").toString();
    m_timer_period = settings->value("Main/timer_period", 5).toInt();
    m_mpd_settings.hostname = settings->value("MPD/hostname", "127.0.0.1");
    m_mpd_settings.port = settings->value("MPD/port", "6600").toInt();
    m_mpd_settings.password = settings->value("MPD/pass", "");


    delete settings;

}

int NowListeningPluginSystem :: activatePlugin(){

    if ( m_plugin_active ) {
        deactivatePlugin();
        activatePlugin();
        return 0;
    }
    if (!m_settings_loaded) loadSettings();
    bool can_be_started = false;

    if (!m_global_settings.global) return 0;

    if ( m_global_mode ){
        if ( m_global_settings.icq_status || m_global_settings.jabber_status){
            can_be_started = true;
            if (m_global_settings.icq_status) m_is_icq_account_active = true;
        }
    }
    else {
        if ( m_icq_account_settings.size() ){
            for ( int i = 0; i < m_icq_account_settings.size(); i++ ){
                if ( m_icq_account_settings[i].status ){
                    can_be_started = true;
                    m_is_icq_account_active = true;
                    continue;
                }
            }

        }
        if ( !can_be_started && m_mrim_account_settings.size() ){
            for ( int i = 0; i < m_mrim_account_settings.size(); i++ ){
                if ( m_mrim_account_settings[i].status ){
                    can_be_started = true;
                    continue;
                }
            }

        }
        if ( !can_be_started && m_jabber_account_settings.size() ){
            for ( int i = 0; i < m_jabber_account_settings.size(); i++ ){
                if ( m_jabber_account_settings[i].status ){
                    can_be_started = true;
                    continue;
                }
            }
        }
    }

    if ( !isWindowsPlayer(m_player_name) && can_be_started ){
        #ifndef Q_WS_WIN

        m_plugin_active = true;

        if (m_is_icq_account_active){
            m_icq_status_check_timer = new QTimer();
            connect (m_icq_status_check_timer, SIGNAL(timeout()), this, SLOT(checkIcqStatuses()));
            m_icq_status_check_timer->start(5000);
        }

        if (m_player_name == "MPD") mpdDataGetter = new
                 MPDDataGetter(m_mpd_settings.hostname.toByteArray(),
                               m_mpd_settings.port,
                               m_mpd_settings.password.toByteArray());
        else if (m_player_name == "Amarok 1.4") amarok14DataGetter = new Amarok14DataGetter();
        else {
            mprisDataGetter = new MPRISDataGetter(m_player_name, m_timer_period);
            connect(mprisDataGetter, SIGNAL(trackChanged(TrackMetaInfo)),
                    this, SLOT(trackChanged(TrackMetaInfo)));
            connect(mprisDataGetter, SIGNAL(playingStoppedStatusChanged(bool)),
                    this, SLOT(playingStoppedStatusChanged(bool)));
            m_current_track = mprisDataGetter->getTrackMetaInfo();
            if ( !mprisDataGetter->isPlayingStopped() ) setStatuses();
        }


        qDebug()<<"Now Listening: plugin activated";
        #endif

    }

    else if ( isWindowsPlayer(m_player_name) && can_be_started ){
        /*
        if (winampDataGetter) return 0;
        winampDataGetter = new WinampDataGetter;
        m_deamon_started = true;
        m_check_track_timer = new QTimer;
        connect(winampDataGetter,SIGNAL(trackChanged()),SLOT(getInfoFromWinamp()));
        winampDataGetter->reset();
        connect(m_check_track_timer, SIGNAL(timeout()), SLOT(ifTrackChanged()));
        m_check_track_timer->start(m_timer_period * 1000);
        */
    }

    return 0;
}

void NowListeningPluginSystem :: deactivatePlugin(){

    if ( m_plugin_active ){

        m_plugin_active = false;

        if (m_is_icq_account_active){
            m_icq_status_check_timer->stop();
            disconnect (m_icq_status_check_timer, SIGNAL(timeout()), this, SLOT(checkIcqStatuses()));
            delete m_icq_status_check_timer;
        }

        /*
        if (winampDataGetter)
        {
            disconnect(winampDataGetter,SIGNAL(trackChanged()),SLOT(getInfoFromWinamp()));
            delete winampDataGetter;
            winampDataGetter = 0;
        }
        */

        #ifndef Q_WS_WIN
        if (m_player_name == "MPD")  delete mpdDataGetter;
        else if (m_player_name == "Amarok 1.4") delete amarok14DataGetter;
        else {
            disconnect(mprisDataGetter, SIGNAL(trackChanged(TrackMetaInfo)),
                        this, SLOT(trackChanged(TrackMetaInfo)));
            disconnect(mprisDataGetter, SIGNAL(playingStoppedStatusChanged(bool)),
                        this, SLOT(playingStoppedStatusChanged(bool)));
            delete mprisDataGetter;
        }
        #endif

    }
    qDebug()<<"NowListening: plugin deactivated";
    setEmptyStatuses();
}

/*
void NowListeningPluginSystem :: ifTrackChanged(){
#ifndef Q_WS_WIN
    if (m_player_name == "MPD"){
        if (mpdDataGetter->getTitle() != m_current_track.title){
            m_current_track = mpdDataGetter->getTrackMetaInfo();
            setStatuses();
        }
        else if (mpdDataGetter->getArtist() != m_current_track.artist){
            m_current_track = mpdDataGetter->getTrackMetaInfo();
            setStatuses();
        }
        else if (mpdDataGetter->getURI() != m_current_track.uri){
            m_current_track = mpdDataGetter->getTrackMetaInfo();
            setStatuses();
        }
    }
    else if (m_player_name == "Amarok 1.4"){
        if (amarok14DataGetter->getTitle() != m_current_track.title){
            m_current_track = amarok14DataGetter->getTrackMetaInfo();
            setStatuses();
        }
        else if (amarok14DataGetter->getArtist() != m_current_track.artist){
            m_current_track = amarok14DataGetter->getTrackMetaInfo();
            setStatuses();
        }
        else if (amarok14DataGetter->getURI() != m_current_track.uri){
            m_current_track = amarok14DataGetter->getTrackMetaInfo();
            setStatuses();
        }
    }
    else {

        if (mprisDataGetter->getTitle() != m_current_track.title){
            m_current_track = mprisDataGetter->getTrackMetaInfo();
            setStatuses();
        }
        else if (mprisDataGetter->getArtist() != m_current_track.artist){
            m_current_track = mprisDataGetter->getTrackMetaInfo();
            setStatuses();
        }
        else if (mprisDataGetter->getURI() != m_current_track.uri){
            m_current_track = mprisDataGetter->getTrackMetaInfo();
            setStatuses();
        }
        if (mprisDataGetter->isPlayingStopped()){
            if (m_current_track.is_stopped);
            else {
                m_current_track.is_stopped = true;
                setStatuses();
            }
        }
        else {
            if (m_current_track.is_stopped) {
                m_current_track.is_stopped = false;
                setStatuses();
            }
        }
    }
#else
    if (isWindowsPlayer(m_player_name))
    {
        winampDataGetter->updateTrackInfo();
    }
#endif
}
*/

void NowListeningPluginSystem :: setStatuses(){

    if ( m_icq_account_settings.size() ){
        for ( int i = 0; i < m_icq_account_settings.size(); ++i ){
            setIcqXStatus(i);
        }
    }
    if ( m_mrim_account_settings.size() ){
        for ( int i = 0; i < m_mrim_account_settings.size(); ++i ){
            setMrimXStatus(i);
        }
    }
    if ( m_jabber_account_settings.size() ){
        for ( int i = 0; i < m_jabber_account_settings.size(); ++i ){
            sendJabberTune(i);
        }
    }

}

void NowListeningPluginSystem :: setIcqXStatus(int account_index){

    if ( ( m_global_mode && m_global_settings.icq_status == 1 ) || (!m_global_mode && m_icq_account_settings[account_index].status == 1) ){
        QSettings settings(QSettings::IniFormat, QSettings::UserScope, "qutim/qutim."+m_profile_name+"/ICQ."+m_icq_account_settings[account_index].name, "accountsettings");
        QString status_title = settings.value("xstatus/caption").toString();
        int status_icon = settings.value("xstatus/index").toInt();
        QStringList account;
        account.append(m_icq_account_settings[account_index].name);
        QString status_message;

            status_message = ( m_global_mode ) ?  m_global_settings.icq_message_mask_1 :
                                              m_icq_account_settings[account_index].message_mask_1;
            status_message = status_message.replace("%artist", m_current_track.artist).replace("%title", ( m_current_track.title.isEmpty() ) ? m_current_track.file_name : m_current_track.title).
                         replace("%album", m_current_track.album).replace("%time", m_current_track.time_min).
                         replace("%tracknumber", m_current_track.number).replace("%uri", m_current_track.uri).replace("%20", " ");

        Event e(m_event_icq, 4, &account, &status_icon, &status_title, &status_message);
        emit m_plugin_system->sendEvent(e);

    }
    else if ( ( m_global_mode && m_global_settings.icq_status == 2 ) || (!m_global_mode && m_icq_account_settings[account_index].status == 2) ){
        QSettings settings(QSettings::IniFormat, QSettings::UserScope, "qutim/qutim."+m_profile_name+"/ICQ."+m_icq_account_settings[account_index].name, "accountsettings");
        if ( settings.value("xstatus/index").toInt() == 11 ){
            QString status_title = settings.value("xstatus/caption").toString();
            int status_icon = settings.value("xstatus/index").toInt();
            QStringList account;
            account.append(m_icq_account_settings[account_index].name);
            QString status_message;

                status_message = ( m_global_mode ) ?  m_global_settings.icq_message_mask_2 :
                                                  m_icq_account_settings[account_index].message_mask_2;
                status_message = status_message.replace("%artist", m_current_track.artist).replace("%title", ( m_current_track.title.isEmpty() ) ? m_current_track.file_name : m_current_track.title).
                            replace("%album", m_current_track.album).replace("%time", m_current_track.time_min).
                            replace("%tracknumber", m_current_track.number).replace("%uri", m_current_track.uri).replace("%20", " ");

            Event e(m_event_icq, 4, &account, &status_icon, &status_title, &status_message);
            emit m_plugin_system->sendEvent(e);
        }
    }

}

void NowListeningPluginSystem :: sendJabberTune(int account_index){

    if ( (m_global_mode && m_global_settings.jabber_status) ||
         !m_global_mode && m_jabber_account_settings[account_index].status){
        QString artist = "";
        QString album = "";
        QString uri = "";
        QString title = "";
        int length = -1;
        int rating = -1;
        QString track = "";
        QStringList protocols;
        protocols.append(m_jabber_account_settings[account_index].name);
        QString data;
        data = ( m_global_mode ) ? m_global_settings.jabber_message : m_jabber_account_settings[account_index].message_mask;
        if ( data.contains("artist") ) artist = m_current_track.artist;
        if ( data.contains("length") ) length = m_current_track.time_msec / 100;
        if ( data.contains("album") ) album = m_current_track.album;
        if ( data.contains("title") && !m_current_track.title.isEmpty() ) title = m_current_track.title;
        else if ( data.contains("title") && m_current_track.title.isEmpty() ) title = m_current_track.file_name;
        if ( data.contains("tracknumber") ) track = m_current_track.number;
        if ( data.contains("uri") ) {
            uri = m_current_track.uri;
            uri = uri.replace("%20", " ");
        }

        qDebug()<<"Now Listening: Jabber tune updated";

        Event e(m_event_jabber, 8, &protocols, &artist, &length, &rating, &album, &title, &track, &uri);
        emit m_plugin_system->sendEvent(e);
    }

}

void NowListeningPluginSystem::setMrimXStatus(int account_index){
    if ( ( m_global_mode && m_global_settings.mrim_status == 1 ) || (!m_global_mode && m_mrim_account_settings[account_index].status == 1) ){

        QString status_message;

            status_message = ( m_global_mode ) ?  m_global_settings.mrim_message_mask_1 :
                                              m_mrim_account_settings[account_index].message_mask_1;
            status_message = status_message.replace("%artist", m_current_track.artist).replace("%title", ( m_current_track.title.isEmpty() ) ? m_current_track.file_name : m_current_track.title).
                         replace("%album", m_current_track.album).replace("%time", m_current_track.time_min).
                         replace("%tracknumber", m_current_track.number).replace("%uri", m_current_track.uri).replace("%20", " ");

        QString status_title;
        QString status_description;
        QString account = m_mrim_account_settings[account_index].name;
        int index = getMrimAccountIndexInList(account);
        if ( !m_global_settings.mrim_mode || !m_mrim_account_settings[account_index].mode ){
            status_title = status_message;
            status_description = m_mrim_current_statuses.at(index).status_description;
        }
        else {
            status_description = status_message;
            status_title = m_mrim_current_statuses.at(index).status_title;
        }

        QStringList accounts;
        accounts.append(account);
        Event e(m_event_mrim_status_set_text, 3, &accounts, &status_title, &status_description);

        emit m_plugin_system->sendEvent(e);

    }
    else if ( ( m_global_mode && m_global_settings.mrim_status == 2 ) || (!m_global_mode && m_mrim_account_settings[account_index].status == 2) ){

        QString account = m_mrim_account_settings[account_index].name;

        int index = getMrimAccountIndexInList(account);
        if (m_mrim_current_statuses.at(index).status_num == 4 && m_mrim_current_statuses.at(index).status_text == "status_53"){
            QString status_message;

                status_message = ( m_global_mode ) ?  m_global_settings.mrim_message_mask_2 :
                                                  m_mrim_account_settings[account_index].message_mask_2;
                status_message = status_message.replace("%artist", m_current_track.artist).replace("%title", ( m_current_track.title.isEmpty() ) ? m_current_track.file_name : m_current_track.title).
                             replace("%album", m_current_track.album).replace("%time", m_current_track.time_min).
                             replace("%tracknumber", m_current_track.number).replace("%uri", m_current_track.uri).replace("%20", " ");

            QString status_title;
            QString status_description;
            if ( !m_global_settings.mrim_mode || !m_mrim_account_settings[account_index].mode ){
                status_title = status_message;
                status_description = m_mrim_current_statuses.at(index).status_description;
            }
            else {
                status_description = status_message;
                status_title = m_mrim_current_statuses.at(index).status_title;
            }

            QStringList accounts;
            accounts.append(account);

            Event e(m_event_mrim_status_set_text, 3, &accounts, &status_title, &status_description);

            emit m_plugin_system->sendEvent(e);
        }

    }
}

void NowListeningPluginSystem::writeCurrentMrimStatus(QString account, qint32 status_num,
                                                      QString status_text, QString status_title, QString status_description){
    if (!m_mrim_current_statuses.size()){
        MrimCurrentStatus temp;
        temp.account = account;
        temp.status_num = status_num;
        temp.status_text = status_text;
        temp.status_title = status_title;
        temp.status_description = status_description;
        m_mrim_current_statuses.append(temp);
    }
    else {
        int index = getMrimAccountIndexInList(account);
        m_mrim_current_statuses[index].status_num = status_num;
        m_mrim_current_statuses[index].status_text = status_text;
        m_mrim_current_statuses[index].status_title = status_title;
        m_mrim_current_statuses[index].status_description = status_description;
    }
}

int NowListeningPluginSystem::getMrimAccountIndexInList(QString account){
    for (int i=0; i < m_mrim_current_statuses.size(); i++){
        if ( m_mrim_current_statuses.at(i).account == account ) return i;
    }
    MrimCurrentStatus temp;
    temp.account = account;
    temp.status_num = 1;
    temp.status_text = temp.status_title = temp.status_description = "";
    m_mrim_current_statuses.append(temp);
    return (m_mrim_current_statuses.size() - 1);
}

void NowListeningPluginSystem::getInfoFromWinamp()
{
    m_current_track = winampDataGetter->getTrackMetaInfo();
    setStatuses();
}

bool NowListeningPluginSystem::isWindowsPlayer(QString player)
{
    return (player.contains("Winamp") || player.contains("AIMP"));
}

void NowListeningPluginSystem::trackChanged(TrackMetaInfo meta_info){
    m_current_track = meta_info;
    setStatuses();
}

void NowListeningPluginSystem::playingStoppedStatusChanged(bool is_stopped){
    if ( is_stopped ){
        qDebug()<<"!!!! Intering in pluginsystem playingStoppedStatusChanged";
        qDebug()<<"!!!! is_stopped = "<<is_stopped;
        //For some players, as Amarok 2 or QMMP need double check
        if (m_player_name == "Amarok 2" || m_player_name == "QMMP"){
            if (mprisDataGetter->isPlayingStopped()) setEmptyStatuses();
        }
        else setEmptyStatuses();
    }
    else setStatuses();
}

void NowListeningPluginSystem::setEmptyStatuses(){
    qDebug()<<"!!!! Setting empty statuses";
    m_global_settings.jabber_message = "";
    m_global_settings.icq_message_mask_1 = "";
    m_global_settings.icq_message_mask_2 = "";
    m_global_settings.mrim_message_mask_1 = "";
    m_global_settings.mrim_message_mask_2 = "";
    for (int i = 0; i < m_icq_account_settings.size(); i++){
        m_icq_account_settings[i].message_mask_1 = "";
        m_icq_account_settings[i].message_mask_2 = "";
    }
    for (int i = 0; i < m_mrim_account_settings.size(); i++){
        m_mrim_account_settings[i].message_mask_1 = "";
        m_mrim_account_settings[i].message_mask_2 = "";
    }
    for (int i = 0; i < m_jabber_account_settings.size(); i++){
        m_jabber_account_settings[i].message_mask = "";
    }
    setStatuses();
    loadSettings();
}

void NowListeningPluginSystem::checkIcqStatuses(){
    for (int i = 0; i < m_icq_account_settings.size(); ++i){
        if ((!m_global_settings.global && m_icq_account_settings.at(i).status) ||
            (m_global_settings.global && m_global_settings.icq_status)){
            QString name = m_icq_account_settings.at(i).name;

            QSettings* accountsettings = new QSettings(QSettings::defaultFormat(), QSettings::UserScope,
                       "qutim/qutim."+m_profile_name+"/ICQ."+name, "accountsettings");
            int temp = accountsettings->value("xstatus/index", 11).toInt();
            int status = m_current_icq_status->value(name);
            if (temp != status){
                if ((!m_global_settings.global && m_icq_account_settings.at(i).status == 2) ||
                    (m_global_settings.global && m_global_settings.icq_status == 2)){
                    if (status == 11){
                        accountsettings->setValue("xstatus10/message", "");
                    }
                    if (temp == 11){
                        setIcqXStatus(i);
                    }
                    m_current_icq_status->insert(name,temp);
                }
                else {
                    QVariant status_2 = status-1;
                    accountsettings->setValue("xstatus"+status_2.toString()+"/message", "");
                    m_current_icq_status->insert(name,temp);
                    setIcqXStatus(i);
                }

            }
        }
    }
}

void NowListeningPluginSystem::managePluginClicked(){
    if (m_plugin_active){
        deactivatePlugin();
        m_manage_plugin->setText(tr("Activate Now Listening"));
    }
    else {
        activatePlugin();
        m_manage_plugin->setText(tr("Deactivate Now Listening"));
    }
}

Q_EXPORT_PLUGIN2 ( NowListeningPluginSystem, NowListeningPluginSystem );
